@@ -18,7 +18,7 @@ pub(crate) use self::{
     comparator::comparator,
     index::{index, slice},
     path::{del_paths, get_path, set_path},
-    string::{explode, implode, stringifier, text},
+    string::{stringifier, text},
     unary::unary,
 };
 
@@ -29,6 +29,7 @@ mod index;
 mod math;
 mod path;
 mod string;
+mod time;
 mod unary;
 
 static INTRINSICS0: phf::Map<&'static str, NamedFn0> = phf_map! {
@@ -40,8 +41,15 @@ static INTRINSICS0: phf::Map<&'static str, NamedFn0> = phf_map! {
     "reverse" => NamedFn0 { name: "reverse", func: reverse },
     "tostring" => NamedFn0 { name: "tostring", func: text },
     "tonumber" => NamedFn0 { name: "to_number", func: string::to_number },
-    "explode" => NamedFn0 { name: "explode", func: explode },
-    "implode" => NamedFn0 { name: "implode", func: implode },
+    "fromjson" => NamedFn0 { name: "fromjson", func: string::from_json },
+    "tojson" => NamedFn0 { name: "tojson", func: string::to_json },
+    "explode" => NamedFn0 { name: "explode", func: string::explode },
+    "implode" => NamedFn0 { name: "implode", func: string::implode },
+
+    "gmtime" => NamedFn0 { name: "gmtime", func: time::gm_time },
+    "localtime" => NamedFn0 { name: "localtime", func: time::gm_time_local },
+    "mktime" => NamedFn0 { name: "mktime", func: time::mk_time },
+    "now" => NamedFn0 { name: "now", func: time::now },
 
     "nan" => NamedFn0 { name: "nan", func: math::nan },
     "infinite" => NamedFn0 { name: "infinite", func: math::infinite },
@@ -51,19 +59,24 @@ static INTRINSICS0: phf::Map<&'static str, NamedFn0> = phf_map! {
     "isfinite" => as_math_fn!(is_finite),
     "isnormal" => as_math_fn!(is_normal),
 };
-static INTRINSICS1: phf::Map<&'static str, (NamedFn1, ArgType)> = phf_map! {
-    "error" => (NamedFn1 { name: "error", func: error1 }, ArgType::Value),
-    "has" => (NamedFn1 { name: "has", func: has }, ArgType::Value),
-    "in" => (NamedFn1 { name: "in", func: |i, c| has(c, i) }, ArgType::Value),
-    "contains" => (NamedFn1 { name: "contains", func: contains }, ArgType::Value),
-    "inside" => (NamedFn1 { name: "inside", func: |i, c| contains(c, i) }, ArgType::Value),
-    "indices" => (NamedFn1 { name: "indices", func: indices }, ArgType::Value),
-    "startswith" => (NamedFn1 { name: "startswith", func: starts_with }, ArgType::Value),
-    "endswith" => (NamedFn1 { name: "endswith", func: ends_with }, ArgType::Value),
-    "delpaths" => (NamedFn1 { name: "delpaths", func: path::del_paths }, ArgType::Value),
+static INTRINSICS1: phf::Map<&'static str, NamedFn1> = phf_map! {
+    "error" => NamedFn1 { name: "error", func: error1 },
+    "has" => NamedFn1 { name: "has", func: has },
+    "in" => NamedFn1 { name: "in", func: |i, c| has(c, i) },
+    "contains" => NamedFn1 { name: "contains", func: contains },
+    "inside" => NamedFn1 { name: "inside", func: |i, c| contains(c, i) },
+    "indices" => NamedFn1 { name: "indices", func: indices },
+    "startswith" => NamedFn1 { name: "startswith", func: starts_with },
+    "endswith" => NamedFn1 { name: "endswith", func: ends_with },
+    "delpaths" => NamedFn1 { name: "delpaths", func: path::del_paths },
+    "bsearch" => NamedFn1 { name: "bsearch", func: binary_search },
+
+    "strftime" => NamedFn1 { name: "strftime", func: time::format_time },
+    "strptime" => NamedFn1 { name: "strptime", func: time::parse_time },
+    "strflocaltime" => NamedFn1 { name: "strflocaltime", func: time::format_time_local },
 };
-static INTRINSICS2: phf::Map<&'static str, (NamedFn2, ArgType, ArgType)> = phf_map! {
-    "setpath" => (NamedFn2 { name: "setpath", func: path::set_path }, ArgType::Value, ArgType::Value),
+static INTRINSICS2: phf::Map<&'static str, NamedFn2> = phf_map! {
+    "setpath" => NamedFn2 { name: "setpath", func: path::set_path },
 };
 
 pub(crate) fn lookup_intrinsic_fn(
@@ -78,12 +91,14 @@ pub(crate) fn lookup_intrinsic_fn(
         INTRINSICS1
             .get(&ident.0)
             .cloned()
-            .map(|(f, t)| (ByteCode::Intrinsic1(f), vec![t]))
+            .map(|f| (ByteCode::Intrinsic1(f), vec![ArgType::Value]))
     } else if *n_args == 2 {
-        INTRINSICS2
-            .get(&ident.0)
-            .cloned()
-            .map(|(f, t1, t2)| (ByteCode::Intrinsic2(f), vec![t1, t2]))
+        INTRINSICS2.get(&ident.0).cloned().map(|f| {
+            (
+                ByteCode::Intrinsic2(f),
+                vec![ArgType::Value, ArgType::Value],
+            )
+        })
     } else {
         None
     }
@@ -243,11 +258,13 @@ pub(crate) fn group_by(context: Value) -> Result<Value> {
 
 fn indices(context: Value, s: Value) -> Result<Value> {
     let ret = match (context, s) {
-        (Value::String(lhs), Value::String(rhs)) => Array::from_vec(
+        (Value::String(lhs), Value::String(rhs)) => Array::from_vec(if rhs.is_empty() {
+            vec![]
+        } else {
             lhs.match_indices(rhs.as_ref())
                 .map(|(pos, _)| Value::Number(pos.into()))
-                .collect(),
-        )
+                .collect()
+        })
         .into(),
         (Value::Array(lhs), Value::Array(rhs)) => Array::from_vec(if rhs.is_empty() {
             vec![]
@@ -284,3 +301,13 @@ fn ends_with(context: Value, s: Value) -> Result<Value> {
         (context, _) => Err(QueryExecutionError::InvalidArgType("endswith", context)),
     }
 }
+
+fn binary_search(context: Value, x: Value) -> Result<Value> {
+    match context {
+        Value::Array(arr) => Ok(match arr.binary_search(&x) {
+            Ok(i) => Value::number(i),
+            Err(i) => Value::number(-1 - (i as isize)),
+        }),
+        _ => Err(QueryExecutionError::InvalidArgType("bsearch", context)),
+    }
+}
