@@ -17,7 +17,6 @@ use nom::{
     sequence::{delimited, pair, preceded, separated_pair, terminated, tuple},
     Finish, IResult, Parser,
 };
-use std::convert::TryFrom;
 
 pub type ParseResult<'a, T> = IResult<&'a str, T>;
 
@@ -283,7 +282,10 @@ fn constant_object(input: &str) -> ParseResult<ConstantObject> {
     delimited(
         terminated(char('{'), multispace0),
         map(separated_list0(ws(char(',')), entry), ConstantObject),
-        preceded(multispace0, char('}')),
+        preceded(
+            multispace0,
+            preceded(opt(terminated(char(','), multispace0)), char('}')),
+        ),
     )(input)
 }
 
@@ -403,7 +405,10 @@ fn term(input: &str) -> ParseResult<Term> {
                 delimited(
                     terminated(char('{'), multispace0),
                     separated_list0(char(','), ws(object_term_entry)),
-                    preceded(multispace0, char('}')),
+                    preceded(
+                        multispace0,
+                        preceded(opt(terminated(char(','), multispace0)), char('}')),
+                    ),
                 ),
                 Term::Object,
             ),
