@@ -59,6 +59,10 @@ pub enum QueryExecutionError {
     InvalidArgType(&'static str, Value),
     #[error("`{0:?}` can't be parsed as a number")]
     InvalidStringToNumber(RcString),
+    #[error("Given string was invalid as a json `{0:}`")]
+    InvalidJson(RcString),
+    #[error("Unable to parse date time")]
+    InvalidDateTimeString(#[from] chrono::format::ParseError),
     #[error("{0:?}")]
     UserDefinedError(String),
 }
